<?php
/***************************************************************************
 *   Copyright (C) 2005-2008 by Ferenc Veres							   *
 *   lion@netngine.hu													   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
if(empty($argc))
{
	exit;
}

class MainWindow extends QWidget
{
	public $backend;
	
	// Main window stuff
	public $vboxMain;	// Main vertical layout
	public $splitterMain;
	public $vboxWorkArea;
	public $vboxSearchArea;
	public $wWorkArea;
	public $wSearchArea;

	public $gridWorkFields;
	
	// Translator text controls
	public $lblLevelsLabel;
	public $lblLevels;
	public $lblOriginal;
	public $txtOriginal;
	public $lblTranslation;
	public $txtTranslation;

	// Translator buttons
	public $btnSaveNext;
	public $btnSaveNextMissing;
	public $btnSave;
	public $btnNext;
	public $btnNextMissing;
	public $btnSavePopStack;
	public $btnSaveToAllChecked;
	
	public $hboxButtons;// Placholder for translator buttons.
	
	public $btnQuit;

	// Search controls
	public $sldPercent;
	public $lcdPercent;
	public $btnRefresh;
	public $hboxSearchPercent; // Placholder for guess controls.

	// Search results
	public $tblSearchResults;
	public $hboxSearchFullView;
	public $lblSearchOriginalFull;
	public $lblSearchTranslationFull;

	// Project tree
	public $modelProjects;
	public $treeProjects;

	// Slots and Signals
	private $slots = array(
		"tblSearchResults_DoubleClicked(int,int)",
		"tblSearchResults_Clicked(int,int)",
		"btnSave_Clicked()",
		"btnSaveNext_Clicked()",
		"btnSaveNextMissing_Clicked()",
		"btnNext_Clicked()",
		"btnNextMissing_Clicked()");
	private $signals = array("");

	// Misc private variables
	private $TableColumnsFixed;
	
	function __construct()
	{
		parent::__construct();

		$this->backend = new QtBackend($this);
		$this->backend->xProjectID = 1;
		$this->backend->xStringID = 0;
		//$backend->EventHandler();

		$this->resize(600,500);

		$this->splitterMain = new QSplitter();
		$this->splitterMain->setOrientation(Qt::Vertical);
		
		$this->vboxMain = new QVBoxLayout();

		// Translator input fields

		$this->gridWorkFields = new QGridLayout();
		$this->gridWorkFields->setMargin(0);
		
		// Project path display
		$this->lblLevelsLabel = new QLabel("Project Path:");
		$this->lblLevels = new QLabel("none");
		$this->gridWorkFields->addWidget($this->lblLevelsLabel, 0, 0);
		$this->gridWorkFields->addWidget($this->lblLevels, 0, 1);

		// Original string
		$this->lblOriginal = new QLabel("Original");
		$this->txtOriginal = new QLabel("");
		$this->txtOriginal->setWordWrap(true);
		//$this->txtOriginal->setTextFormat(Qt::PlainText);
		$this->txtOriginal->setMinimumHeight(70);
		$this->gridWorkFields->addWidget($this->lblOriginal, 1,0);
		$this->gridWorkFields->addWidget($this->txtOriginal, 1,1);
		
		// Translaition field
		$this->lblTranslation = new QLabel("Translation");
//		$this->txtTranslation = new QTextEdit("a");
//		$this->txtOriginal->setMaximumHeight(70);
		$this->gridWorkFields->addWidget($this->lblTranslation, 2,0);
//		$this->gridWorkFields->addWidget($this->txtTranslation, 2,1);

		// Translator buttons
		$this->hboxButtons = new QHBoxLayout();
		$this->hboxButtons->setMargin(0);
		
		$this->btnSaveNext = new QPushButton("Save + Next");
		$this->hboxButtons->addWidget($this->btnSaveNext);
		$this->btnSaveNextMissing = new QPushButton("Save + Next missing");
		$this->hboxButtons->addWidget($this->btnSaveNextMissing);
		$this->btnSave = new QPushButton("Save");
		$this->hboxButtons->addWidget($this->btnSave);
		$this->btnNext = new QPushButton("Next");
		$this->hboxButtons->addWidget($this->btnNext);
		$this->btnNextMissing = new QPushButton("Next missing");
		$this->hboxButtons->addWidget($this->btnNextMissing);
		$this->btnSavePopStack = new QPushButton("Save + Pop");
		$this->hboxButtons->addWidget($this->btnSavePopStack);
		$this->btnSaveToAllChecked = new QPushButton("Save to all checked");
		$this->hboxButtons->addWidget($this->btnSaveToAllChecked);
		$this->hboxButtons->addStretch(1);
		$this->btnQuit = new QPushButton("Quit");
		$this->hboxButtons->addWidget($this->btnQuit);

		// Search controls
		$this->sldPercent = new QSlider();
		$this->sldPercent->setOrientation(Qt::Horizontal);
		$this->sldPercent->setRange(0, 99);
		$this->sldPercent->setValue(50);
		$this->lcdPercent = new QLCDNumber(2);
		$this->lcdPercent->display(50);
		$this->btnRefresh = new QPushButton("Refresh");

		// Layout guess controls
		$this->hboxSearchPercent = new QHBoxLayout();
		$this->hboxSearchPercent->addWidget($this->sldPercent);
		$this->hboxSearchPercent->addWidget($this->lcdPercent);
		$this->hboxSearchPercent->addWidget($this->btnRefresh);
		
		// Full view of search entries
		$this->hboxSearchFullView = new QHBoxLayout();
		$this->lblSearchOriginalFull = new QLabel("1");
		$this->lblSearchOriginalFull->setMinimumHeight(70);
		$this->lblSearchOriginalFull->setWordWrap(true);
		$this->lblSearchTranslationFull = new QLabel("2");
		$this->lblSearchTranslationFull->setMinimumHeight(70);
		$this->lblSearchTranslationFull->setWordWrap(true);
		$this->hboxSearchFullView->addWidget($this->lblSearchOriginalFull);
		$this->hboxSearchFullView->addWidget($this->lblSearchTranslationFull);

		// Search results table
		$this->tblSearchResults = new QTableWidget(0, 3);
		$this->tblSearchResults->setSelectionBehavior(1); //QAbstractItemView::SelectRows);
		$this->tblSearchResults->setEditTriggers(0);
 		$this->tblSearchResults->setHorizontalHeaderItem(0,  new QTableWidgetItem(new QString("Match"), QTableWidgetItem::Type));
 		$this->tblSearchResults->setHorizontalHeaderItem(1,  new QTableWidgetItem(new QString("Original string"), QTableWidgetItem::Type));
 		$this->tblSearchResults->setHorizontalHeaderItem(2,  new QTableWidgetItem(new QString("Translated string"), QTableWidgetItem::Type));

		// Test fails:
		//$this->tblSearchResults->setRowCount(5);
 		//$this->tblSearchResults->setItem(1, 1, new QTableWidgetItem(new QString("Translated string"), QTableWidgetItem::Type));

		// Project tree
		//$this->modelProjects = new ProjectTreeModel();
		//$this->treeProjects = new QTreeView($this);
		//$this->treeProjects->setModel($this->modelProjects);

		//$this->vboxMain->addWidget($this->treeProjects);

		$this->ConnectSignals();
		
		
		// Add controls to the main layout

		$this->wWorkArea = new QWidget();
		$this->vboxWorkArea = new QVBoxLayout($this->wWorkArea);

		$this->vboxWorkArea->addLayout($this->gridWorkFields);
		$this->vboxWorkArea->addLayout($this->hboxButtons);
		
		$this->wSearchArea = new QWidget();
		$this->vboxSearchArea = new QVBoxLayout($this->wSearchArea);

		$this->vboxSearchArea->addLayout($this->hboxSearchFullView);
		$this->vboxSearchArea->addWidget($this->tblSearchResults);
		$this->vboxSearchArea->addLayout($this->hboxSearchPercent);

		$this->splitterMain->addWidget($this->wWorkArea);
		$this->splitterMain->addWidget($this->wSearchArea);

		$this->vboxMain->addWidget($this->splitterMain);
		$this->setLayout($this->vboxMain);

	}

	function ConnectSignals()
	{
		// Translator button events
		$this->connect($this->sldPercent, SIGNAL("valueChanged(int)"),$this->lcdPercent, SLOT("display(int)"));
		$this->connect($this->btnQuit, SIGNAL("clicked()"), QApplication::instance(), SLOT("quit()"));
		$this->connect($this->btnSave, SIGNAL("clicked()"), $this, SLOT("btnSave_Clicked()"));
		$this->connect($this->btnSaveNext, SIGNAL("clicked()"), $this, SLOT("btnSaveNext_Clicked()"));
		$this->connect($this->btnSaveNextMissing, SIGNAL("clicked()"), $this, SLOT("btnSaveNextMissing_Clicked()"));
		$this->connect($this->btnNext, SIGNAL("clicked()"), $this, SLOT("btnNext_Clicked()"));
		$this->connect($this->btnNextMissing, SIGNAL("clicked()"), $this, SLOT("btnNextMissing_Clicked()"));

		// Search results table events
		//$this->connect($this->tblSearchResults, SIGNAL("cellDoubleClicked(int,int)"), $this, SLOT("tblSearchResults_DoubleClicked(int,int)"));
		//$this->connect($this->tblSearchResults, SIGNAL("cellClicked(int,int)"), $this, SLOT("tblSearchResults_Clicked(int,int)"));
	}

	private function FixTableColumns()
	{
		if(!$this->TableColumnsFixed)
		{
			$this->TableColumnsFixed = true;
			$width = $this->tblSearchResults->width();
			$this->tblSearchResults->setColumnWidth(0, (int) round($width/100*10) - 10);
			$this->tblSearchResults->setColumnWidth(1, (int) round($width/100*45) - 10);
			$this->tblSearchResults->setColumnWidth(2, (int) round($width/100*45) - 10);
		}
	}
	
	//function paintEvent(QPaintEvent &event)
	//{
		//echo "hello";
		//$this->painter = new QPainter($this);
		//$this->painter->setBrush(Qt::blue);
		//$this->painter->drawRect(new QRect(30, -5, 20, 10));
	//}
	
	function tblSearchResults_Clicked($y, $x)
	{
		$item = $this->tblSearchResults->item($y, 1);
		print_r($item);
		if($item)
		{
			$this->lblSearchOriginalFull->setText($item->text());
		}
		$item = $this->tblSearchResults->item($y, 2);
		print_r($item);
		if($item)
		{
			$this->lblSearchTranslationFull->setText($item->text());
		}
		echo "Click: X:$x, Y:$y\n";
	}
	
	function tblSearchResults_DoubleClicked($x, $y)
	{
		echo "DoubleClick: X:$x, Y:$y\n";
	}
	
	// Save and Navigate button event handlers
	
	function btnSave_Clicked()
	{
		$this->SaveNavigate(true, LOCATE_CURR);
	}

	function btnSaveNext_Clicked()
	{
		$this->SaveNavigate(true, LOCATE_NEXT);
	}

	function btnSaveNextMissing_Clicked()
	{
		$this->SaveNavigate(true, LOCATE_NEXTMISSING);
	}

	function btnNext_Clicked()
	{
		$this->SaveNavigate(false, LOCATE_NEXT);
	}

	function btnNextMissing_Clicked()
	{
		$this->SaveNavigate(false, LOCATE_NEXTMISSING);
	}

	// Local functions for saving and searching
	
	private function SaveNavigate($save, $locType)
	{
		$this->FixTableColumns();
		
		if($save)
		{
			echo "NOT SAVING YET";
		}

		$this->backend->LocateString($locType);
		
		$this->backend->SyncToString();
		if(!empty($this->backend->LocatedString))
		{
			$this->txtOriginal->setText(new QString($this->backend->LocatedString->GetOriginal()));
			//$this->txtTranslation->setText(QString::fromUtf8($this->backend->LocatedString->GetTranslation()));
			
			$this->backend->ddlSearchType = 0;
			$this->backend->txtFindText = $this->backend->LocatedString->GetOriginal();
			$this->backend->FindStrings();

			$this->ShowFindResults();
			$this->ShowProjectPath();
		}
	}
	 
	private function ShowFindResults()
	{
		if(!empty($this->backend->Translations))
		{
			$filterPercent = 60;
			$localeID = 1;
	
			// Count necessary rows and resize the search results table
			$rows = 0;
			foreach($this->backend->Translations as $id => $tr)
			{
				$rows += count($tr["strings"]);
			}
			$this->tblSearchResults->setRowCount($rows);

			$i = 0;
			foreach($this->backend->Translations as $id => $tr)
			{
				foreach($tr["strings"] as $key => $str)
				{
					$txt = new QString($tr["percent"]."%");
					$newItem = new QTableWidgetItem($txt, QTableWidgetItem::Type);
					//$newItem->setFlags(1 | 32); //Qt::ItemIsSelectable | Qt::ItemIsEnabled);
					$this->tblSearchResults->setItem($i, 0, $newItem);

					$txt = new QString($str->GetOriginal());
					$newItem = new QTableWidgetItem($txt, QTableWidgetItem::Type);
					$this->tblSearchResults->setItem($i, 1, $newItem);

					$newItem = new QTableWidgetItem(QString::fromUtf8($str->GetTranslation(), -1), QTableWidgetItem::Type);
					$this->tblSearchResults->setItem($i, 2, $newItem);
					$i++;
				}
			}
		}
	}

	private function ShowProjectPath()
	{
		$path = "Project Path: ";
		
		// Display project path
		if(!empty($this->backend->LocatedProjects))
		{
			//$levelIndex = 0;
			for($i = count($this->backend->LocatedProjects)-1; $i >= 0; $i--)
			{
				// Display project path entries
				//$this->btnLevels[$levelIndex]->setText(new QString($this->backend->LocatedProjects[$i]->GetName()));
				//$this->btnLevels[$levelIndex]->setVisible(true);
				//$levelIndex++;
				
				$path .= $this->backend->LocatedProjects[$i]->GetName();
				if($i != 0)
				{
					$path .= " -> ";
				}
			}

			// Hide rest of the items
			//for($i = $levelIndex; $i<10; $i++)
			//{
			//	$this->btnLevels[$i]->setVisible(false);
			//}
		}
		$this->lblLevels->setText(QString::fromUtf8($path));
	}

}



class MainWindowLater extends QMainWindow
{
	public $windowCentral;

	public $fileMenu;
	public $editMenu;
	public $helpMenu;
	public $newAct;
	
	function __construct()
	{
		parent::__construct();

		$this->windowCentral = new WindowCentral();
		$this->setCentralWidget($this->windowCentral);
		//$statusBar = $this->statusBar();
		//$this->setStatusBar($this->statusBar());

		$this->CreateMenus();
	}

	function CreateMenus()
	{
		//$menuBar = ();
		$this->fileMenu = $this->menuBar()->addMenu("&File");
		//$this->fileMenu->addAction(openAct);
		//$this->fileMenu->addAction(saveAct);
		//$this->fileMenu->addAction(saveAsAct);
		//$this->fileMenu->addSeparator();
		//$this->fileMenu->addAction(exitAct);

		//$this->editMenu = $this->menuBar()->addMenu("&Edit");
		//editMenu->addAction(cutAct);
		//editMenu->addAction(copyAct);
		//editMenu->addAction(pasteAct);

		//$this->menuBar()->addSeparator();

		//$this->helpMenu = $this->menuBar()->addMenu("&Help");
		//helpMenu->addAction(aboutAct);
		//helpMenu->addAction(aboutQtAct);
	}
}


?>